﻿using PT.Scheduler;
using PT.Scheduler.Simulation.Customizations;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using PT.Common;
using System.Runtime.ExceptionServices;

namespace PlanetTogether.Sample.SuccessorMOOverlap
{
    public class Class1 : SchedulabilityCustomization
    {
        public override string Name => "SuccessorMOOverlap";

        public override string Description => "Sample that allows a successor order to overlap the predecessor after the User Defined SuccessorMOTransferHrs ";

        

        // This function checks to see if the MO exists. If the MO exists it checks to see if the UDF is defined.
        protected void MOExists(ScenarioDetail a_sd, InternalActivity a_activity, out ManufacturingOrder o_MO, out BaseOperation o_op)
        {
            o_MO = null;
            o_op = null;
            // Checks the UDF value and if it is null.
            if (a_activity.Operation.UserFields?.Find("SuccessorMO")?.DataValue is string UDFValue)
            {
                // The program expects the UDF input will be seperated by ',' ie. MO010, Op010
                string[] UDFArray = UDFValue.Split(',');
                //UDFArray = UDFValue.Split(',');
                Job job = a_sd.JobManager.GetById(a_activity.Operation.ManufacturingOrder.Job.Id);

                // If job.ManufacturingOrder is null then this if statement is skipped and returns false.
                if (job.ManufacturingOrders?.GetByExternalId(UDFArray[0]) is ManufacturingOrder MO)
                {

                    o_MO = MO;
                    for (int i = 0; i < MO.OperationCount; i++)
                    {
                        BaseOperation baseOperation = MO.OperationManager.GetByIndex(i);
                        if (baseOperation.ExternalId == UDFArray[1])
                        {
                            // Sets the out parameter to baseOperation
                            o_op = baseOperation;
                        }
                    }
                }
            }
        }

        // This function checks if the successor order is schedulable
        protected override bool IsSchedulable(ScenarioDetail a_sd, ScenarioDetail.SimulationType a_simType, long a_clock, long a_simulationClock, BaseResource a_res, BaseActivity a_activity, out long o_postEventTime)
        {
            o_postEventTime = -1;
            InternalActivity activity1 = a_activity as InternalActivity;

            MOExists(a_sd, activity1, out ManufacturingOrder predecessorMo, out BaseOperation predecessorOp);

            if (predecessorMo == null || predecessorOp == null)
            {
                return true;
            }
            long scheduleDelay = Schedule(predecessorOp, activity1, a_simulationClock);
            if (scheduleDelay == 0)
            {
                return true;
            }
            o_postEventTime = scheduleDelay;
            return false;
        }

        // Function returns earliest schedulable date of successor
        protected long Schedule(BaseOperation a_predecessorOperation, InternalActivity a_activity, long a_simulationClock)
        {
            // First checks if the predecessor is scheduled
            if (a_predecessorOperation.Scheduled)
            {
                // If UDF SuccessorMOTransferHrs is defined it gets the value and sets it to UDFHrsValue
                if (a_activity.Operation.UserFields?.Find("SuccessorMOTransferHrs")?.DataValue is double UDFHrsValue)
                {
                    // If 
                    if (a_predecessorOperation.ScheduledStartDate.AddHours(UDFHrsValue).Ticks <= a_simulationClock)
                    {
                        return 0;
                    }

                    return a_predecessorOperation.ScheduledStartDate.AddHours(UDFHrsValue).Ticks;
                }
                // If UDF doesn't exist or is equal to 0.
                else
                {
                    //
                    if (a_predecessorOperation.ScheduledEndDate.Ticks <= a_simulationClock)
                    {
                        return 0;
                    }
                    //
                    return a_predecessorOperation.ScheduledEndDate.Ticks;
                }
            }
            // If predecessor is not scheduled
            else
            {
                if (a_activity.Operation.UserFields?.Find("SuccessorMOTransferHrs")?.DataValue is double UDFHrsValue)
                {
                    return new DateTime(a_simulationClock).AddHours(UDFHrsValue).Ticks;
                }
                // Might need to account for ProductRules, efficiency multipliers, etc.
                return a_simulationClock + a_predecessorOperation.WorkContent.Ticks;
            }

        }
    }
}
